@@ -1,7 +1,7 @@
 //! Functions implementing sRGB gamma compression and expansion formulæ.
 
-const S_0: f32 = 0.00313066844250060782371;
-const E_0: f32 = 12.92 * S_0;
+// Defines S_0 and E_0 constants
+include!(concat!(env!("OUT_DIR"), "/gamma_constants.rs"));
 
 macro_rules! expand_impl {
     ($e:ident, $t:ty, $low:expr, $high:expr) => {{
@@ -29,7 +29,7 @@ macro_rules! compress_impl {
         // Adding 0.5 is for rounding.
         (if $s <= S_0 {
             const D: f32 = 12.92 * RANGE;
-            $s.max(0.0).mul_add(D, 0.5)
+            crate::maths::mul_add($s.max(0.0), D, 0.5)
         } else {
             const A: f32 = 0.055 * RANGE;
             const D: f32 = 1.055 * RANGE;
@@ -223,7 +223,7 @@ pub fn expand_normalised(e: f32) -> f32 {
 ///
 /// ```
 /// assert_eq!(0.0,        srgb::gamma::compress_normalised(0.0));
-/// assert_eq!(0.23921567, srgb::gamma::compress_normalised(0.046665084));
+/// assert_eq!(0.23921569, srgb::gamma::compress_normalised(0.046665084));
 /// assert_eq!(0.91372544, srgb::gamma::compress_normalised(0.8148465));
 /// // Unfortunately, imprecision of floating point numbers may be an issue:
 /// assert_eq!(0.99999994, srgb::gamma::compress_normalised(1.0));
@@ -325,7 +325,7 @@ pub fn linear_from_normalised(normalised: [f32; 3]) -> [f32; 3] {
 ///     srgb::gamma::normalised_from_linear([0.8148467, 0.80695236, 0.79910284])
 /// );
 /// assert_eq!(
-///     [0.83137256, 0.1294117, 0.23921564],
+///     [0.83137256, 0.12941168, 0.23921566],
 ///     srgb::gamma::normalised_from_linear([0.65837485, 0.0152085, 0.04666508])
 /// );
 /// ```
